from flask import render_template_string, Markup

def render_field(field, **kwargs):
    """
    Returns a rendered field using the boilerplate's field renderer.

    :param field: A wtforms.field to render
    :param kwargs: Keyword arguments to send to the render macro. allows 
                   renderer configuration when calling this global within 
                   a template.
    """

    template = """
    {% from '_boilerplate/macros.html' import render_field %}
    {{ render_field(field, **kwargs) }}
    """
    return Markup(render_template_string(template, field=field, kwargs=kwargs))

def csrf_setup(**kwargs):
    """
    Return the CSRF setup global for injection into jinja templates.
    """

    template = """
    {% from '_boilerplate/macros.html' import csrf_setup %}
    {{ csrf_setup(**kwargs) }}
    """
    return Markup(render_template_string(template, kwargs=kwargs))


from flask.ext.classy import FlaskView
from flask import g

class MenuFlaskView(FlaskView):
    @classmethod
    def register(cls, app):
        """
        Take the cached attributes from the methods that were
        created using the menu_item decorator. 

        Turn them into a list and sort it.
        """

        super(__class__, cls).register(app)
        cls._menu_items = []
        for meth_str in dir(cls):
            meth = getattr(cls, meth_str)
            if hasattr(meth, '_menu_items'):
                href = '{}:{}'.format(cls.__name__, meth.__name__)
                for menu_item in meth._menu_items:
                    menu_item = menu_item + (href,)
                    cls._menu_items.append(menu_item)
        cls._menu_items.sort(key=lambda x: x[2])

    def before_request(self, name, **kwargs):
        """
        Make the menu items accessible to the template renderer.
        Currently there is no context_preprocessor available for 
        FlaskView's, so this is as good as it gets.
        """
        g.menu_items = self._menu_items

def menu_item(title, identifier=None, position=0):
    def decorator(f):
        item = (title, identifier or f.__name__, position)
        if not hasattr(f, '_menu_items') or f._menu_items is None:
            f._menu_items = [item]
        else:
            f._menu_items.append(item)
        return f
    return decorator



import os
from setuptools import setup

readme_path = os.path.join(os.path.dirname(
  os.path.abspath(__file__)),
  'README.rst',
)
long_description = open(readme_path).read()
version_path = os.path.join(os.path.dirname(
  os.path.abspath(__file__)),
  'VERSION',
)
version = open(version_path).read()


setup(
  name='flask-boilerplate-utils',
  version=version,
  packages=['flask_boilerplate_utils'],
  author="Nick Whyte",
  author_email='nick@nickwhyte.com',
  description="Additional functionality with easy upgrading for the flask-boilerplate.",
  long_description=long_description,
  url='https://github.com/nickw444/flask-boilerplate-utils',
  include_package_data=True,
  zip_safe=False,
  install_requires=[
    "Flask",
    "raven",
    "flask-wtf",
    "flask-babel",
    "flask-script",
    "meinheld",
    "flask-classy"
  ],
)
